#!/usr/bin/ksh
#set -x
#
#=============================================================================
# Nom            :      7upacl.ksh
# Projet         :      
#
# Repertoires    :      script -> /usr/local/bin
#
# Cree le        :      16/10/2001
# Version        :      2.00
# Cree par       :      Jean Philippe BOCQUENET
# Objet          :      Positionne les ACLs et les Users/Groups
# Appel(s)       :      commande en ligne
# Environnements :      Solaris 5.6
# Parametre(s)   :      Positionne les ACLs et les Users/Groups pour les projets
#                       				1) nom du projet
#				                        2) flag verbeux (optionel) 'v' ou ''
#				                        3) nom du sous repertoire (optionel)
# Information    :      Si il existe, le fichier 'nom_projet'.README est edite
#				                (dans le repertoire /admin/tools/compte-projet)
# Issues         :      Les fichiers ACL se trouvent dans /sys_admin/projects
# Codes retour   :      = 0 => OK
#                :      = 1 => KO
#=============================================================================
# Modification(s)
#-----------------------------------------------------------------------------
# JJ/MM/AA - QUI
#       QUOI
#
#=============================================================================

 
# Nom du script shell
#-----------------------------------------------------------------------------
#
PGM=`basename $0`

# Trace de l'execution
#-----------------------------------------------------------------------------
#
REP_LOG=/tmp
FIC_LOG=`echo $PGM | cut -f 1 -d "."`.log
#exec </dev/null >>$REP_LOG/$FIC_LOG 2>&1

# Langue
# ------
export LANG=C

# Parameters
# ----------
PROJECT=$1
VERBOSE=$2
SUBDIRECTORY=$3

# Variable
# --------
DATE=`date +%Y%m%d%H%M`

REP_DATA=/export/data

#REP_DATA=/usr/var/ase/mnt/ASEdata/export/data
REP_DATACD=/usr/var/ase/mnt/ASEcd/export/datacd
REP_ACL=/sys_admin/projects
FIC_ACL=${REP_ACL}/acl_${PROJECT}.lis
FIC_README_ACL=${REP_ACL}/${PROJECT}.README

MASK_REP="user::rwx,group::rwx,other::---"
MASK_FIC="user::rw-,group::rw-,other::---"


# Starting verification
# ---------------------
if [ ! -z "${VERBOSE}" ]
then	
	clear
fi
echo "\nProcessing 7upacl.ksh file (V${VERSION}) for ${PROJECT}"

# Parameters test
# ---------------
echo "\nVerifying input parameters: \c"
if [ $# -lt 1 ]
then
	echo "\n\nParameters missing" 
	echo "\nUsage: 7upacl.ksh project_name  [VERBOSE] [SUBDIRECTORY]"
	echo "Example: 7upacl.ksh dio_toulouse V V4"
	exit 1;
else
	if [ -d ${REP_DATA}/${PROJECT} ]
	then
		if [ -d ${REP_DATACD}/${PROJECT} ] 
		then 
			echo "\n\nFAILED : Ambigous Project Name : it exists on both DATA and DATACD !"
			exit 1;	
		else
			REP_HOME=${REP_DATA}
		fi
	else
		if [ -d ${REP_DATACD}/${PROJECT} ]
		then
			REP_HOME=${REP_DATACD}
		else
			echo "\n\nFAILED : Project not found on ${REP_DATA} neither on ${REP_DATACD} !"
			exit 1;
		fi
	fi

	REP_PROJECT=${REP_HOME}/${PROJECT}
	GROUP=`ls -ld ${REP_HOME}/${PROJECT} | awk '{print $4}'`

	if [ ! -f "${FIC_ACL}" ]
	then
		echo "\n\nFAILED : ACL file ${FIC_ACL} not found on ${REP_ACL} !"
		exit 1;
	fi
	if [ ! -z "${VERBOSE}" ]
	then	
		if [ "${VERBOSE}" != "V" ]
		then
			echo "\n\nParameter must be 'V' or '' !!"
			exit 1;
		fi
	fi
	if [ ! -d "${REP_PROJECT}/${SUBDIRECTORY}" ]
	then
		echo "\n\nSub-directory ${SUBDIRECTORY} not found on ${REP_PROJECT} !!"
		exit 1;
	else
		REP_PROJECT=${REP_HOME}/${PROJECT}/${SUBDIRECTORY}
	fi
fi
echo "Ok"
echo "The project home directory is : ${REP_HOME}"
# User/Group test
# ---------------
echo "Verifying acl_${PROJECT}.lis file: \c"
ypcat passwd > /tmp/map_nis_passwd_${DATE}
cat /etc/passwd >> /tmp/map_nis_passwd_${DATE}
ypcat group > /tmp/map_nis_group_${DATE}
cat /etc/group >> /tmp/map_nis_group_${DATE}
while read ITEM;
do
        TYPE=`echo $ITEM | awk -F: '{print $1}'`
        case $TYPE in
        'user')
                ACCOUNT_NAME=`echo $ITEM | awk -F: '{print $2}'`
                UID=`grep "${ACCOUNT_NAME}:" /tmp/map_nis_passwd_${DATE}`
                GID=`grep "${ACCOUNT_NAME}:" /tmp/map_nis_passwd_${DATE} | awk -F: '{print $4}'`
		GID_NAME=`grep "${GID}:" /tmp/map_nis_group_${DATE} | awk -F: '{print $1}'`
                if [ -z "$UID" ]
                then
                        echo "\n\nWARNING: account ${ACCOUNT_NAME} does not exist !!"
                        exit 1;
                fi
		if [ "${GID_NAME}" = "structur" ]
		then
			MASK_REP="user::rwx,group::rwx,other::rwx"
		fi
                ;;
        'group')
                GROUP_NAME=`echo $ITEM | awk -F: '{print $2}'`
                GID=`grep "${GROUP_NAME}:" /tmp/map_nis_group_${DATE}`
                if [ -z "$GID" ]
                then
                        echo "\n\nWARNING: group ${GROUP_NAME} does not exist !!"
                        exit 1;
                fi
                ;;
	'*')
		echo "\n\nWARNING: syntax error on ${FIC_ACL} !!"
		exit 1;
		;;
        esac
done < ${FIC_ACL}
rm /tmp/map_nis_passwd_${DATE}
rm /tmp/map_nis_group_${DATE}
echo "Ok"

# Overview (if VERBOSE mode used)
# -------------------------------
if [ ! -z "${VERBOSE}" ]
then
	echo "\nOverview:"
	echo "---------"
	echo "\nProject name: 		${PROJECT}"
	echo "Root directory: 	${REP_PROJECT}"
	if [ ! -z "${SUBDIRECTORY}" ]
	then
		echo "Sub-directory: 		${SUBDIRECTORY}"
	fi 
	echo "ACLs file (verifyed): 	${FIC_ACL}"
	echo "ACL file content: \n"
	cat ${FIC_ACL}
fi

# Test if restriction exist
# -------------------------
if [ -f ${FIC_README_ACL} ]
then
	echo ""
	cat ${FIC_README_ACL}
	echo "\nWaiting for 15 seconds... \c"
	sleep 15
	echo "Ok"
fi

# Listing of directory
# --------------------
echo "Searching for the directories in ${REP_PROJECT}: \c"
find $REP_PROJECT -type d -print > /tmp/${PROJECT}_${DATE}
echo "Ok"
# find number of directories
# --------------------------
integer NB_REP=0
NB_REP=`wc -l /tmp/${PROJECT}_${DATE} | awk '{print $1}'`

# find number of files
# --------------------
#ITEM=`/usr/sbin/repquota -a -g | grep -w ${GROUP} | awk '{print $6}'`
#if [ "${ITEM}" = "none" ]
#then
#	ITEM=`/usr/sbin/repquota -a -g | grep -w ${GROUP} | awk '{print $7}'`
#fi
#integer NB_FIC
#NB_FIC=${ITEM}
#if [ "${NB_FIC}" != "0" ]
#then
#	NB_FIC=${NB_FIC}-${NB_REP}
#fi

####### Declaring the Function which sets ACL for Files (While Loop)

function PutFilesACL
{
	if [ ! -z "${VERBOSE}" ]
	then
        	echo "\nDirectory: $ITEM"
        	echo "          Setting ACLs files: \c"
	fi
	PWDLENGTH=`expr length "$ITEM"`
	FILE_NAMES_LENGTH=`ls -al $ITEM | grep ^-  | awk '{print $9}' | wc -c`			
	FILENUMBER=`ls -al $ITEM | wc -l | awk '{print $1}'`
	ARGLENGTH=`expr $FILE_NAMES_LENGTH + $FILENUMBER \* $PWDLENGTH`	
	if [ $ARGLENGTH -lt 17500 ]
	then
		setacl -b "${ITEM}/"* > /dev/null 2>&1
		setacl -b "${ITEM}/."??*  > /dev/null 2>&1
		setacl -U $REP_ACL/acl_${PROJECT}.lis "${ITEM}/"* > /dev/null 2>&1
		setacl -U $REP_ACL/acl_${PROJECT}.lis "${ITEM}/."??* > /dev/null 2>&1
		chgrp ${GROUP} "${ITEM}/"* > /dev/null 2>&1
		chgrp ${GROUP} "${ITEM}/."??* > /dev/null 2>&1
	else
		if [ ! -z "${VERBOSE}" ]
		then
			echo "\n          The arg. list is ${ARGLENGTH} characters long. Using enhanced method...\c"
		fi
        	ls -al $ITEM | grep ^- | awk '{print $9}' > /tmp/LISTFILES
        	while read FILE
        	do
                	setacl -b "${ITEM}/${FILE}" 2>> /tmp/${PROJECT}_errors_${DATE}
                	setacl -U $REP_ACL/acl_${PROJECT}.lis "${ITEM}/${FILE}" 2>> /tmp/${PROJECT}_errors_${DATE}
                	chgrp ${GROUP} "${ITEM}/${FILE}" 2>> /tmp/${PROJECT}_errors_${DATE}
        	done < /tmp/LISTFILES
        	rm /tmp/LISTFILES
		if [ ! -z "${VERBOSE}" ]
                then
                        echo "OK"
                fi
	fi
}

######## Declaring the Function which sets the group 
function SetGroup
{
	if [ ! -z "${VERBOSE}" ]
	then
        	echo "Ok"
        	echo "          Setting group ${GROUP}: \c"
	fi
	chgrp ${GROUP} "$ITEM" 2>> /tmp/${PROJECT}_errors_${DATE}
}

######## Declaring the Function which resets the directory ACL 
function ResetDirACL
{
	if [ ! -z "${VERBOSE}" ]
	then
        	echo "Ok"
        	echo "          Reset ACLs directory: \c"
	fi
	setacl -b "$ITEM" 2>> /tmp/${PROJECT}_errors_${DATE}
	setacl -D -b "$ITEM" 2>> /tmp/${PROJECT}_errors_${DATE}
	setacl -d -b "$ITEM" 2>> /tmp/${PROJECT}_errors_${DATE}
}
	
######## Declaring the Function which sets the directory ACL	
function SetDirACL
{
	if [ ! -z "${VERBOSE}" ]
	then
        	echo "Ok"
        	echo "          Setting ACLs directory: \c"
	fi
	setacl -u ${MASK_REP} -U ${FIC_ACL} "$ITEM" 2>> /tmp/${PROJECT}_errors_${DATE}
	setacl -D -u ${MASK_REP} -U ${FIC_ACL} "$ITEM" 2>> /tmp/${PROJECT}_errors_${DATE}
	setacl -d -u ${MASK_FIC} -U ${FIC_ACL} "$ITEM" 2>> /tmp/${PROJECT}_errors_${DATE}
	if [ ! -z "${VERBOSE}" ]
	then
        	echo "Ok"
	fi
}

# Main loop
# ---------
if [ -z "${VERBOSE}" ]
then
	echo "Setting up ACLs/Group (${NB_REP} directories): ... working ... \c"
else
	echo "Setting up ACLs/Group (${NB_REP} directories): "
fi

while read ITEM
do

# Calling the PutFilesACL function
PutFilesACL

# Calling the SetGroup function
SetGroup

# Calling the ResetDirACL function
ResetDirACL

# Calling the SetDirACL function
SetDirACL

done < /tmp/${PROJECT}_${DATE}

if [ -z "${VERBOSE}" ]
then
	echo "Ok"
fi

# Listing of root's files
# -----------------------
echo "Setting up Owner (if root only): \c "
find ${REP_PROJECT} -user root -print > /tmp/${PROJECT}_root_${DATE}

# Changing owner (if it s root)
# -----------------------------
while read ITEM
do
	chown projet "$ITEM"

done < /tmp/${PROJECT}_root_${DATE}

echo "Ok"
  
# Setting up protection of root directory
# ---------------------------------------
if [ -z "${SUBDIRECTORY}" ]
then
	echo "Setting up protection of ${PROJECT}: \c"
	chown projet "$REP_PROJECT"
	chgrp $GROUP "$REP_PROJECT"
	chmod 700 "$REP_PROJECT"
	echo "Ok"
fi

# Checking errors
# ---------------
cp /tmp/${PROJECT}_errors_${DATE} /tmp/debug.$PROJECT
cat /tmp/${PROJECT}_errors_${DATE} | grep -v '\*: ' | grep -v 'Operation not supported on socket' > /tmp/${PROJECT}_errors_${DATE}.tmp
mv /tmp/${PROJECT}_errors_${DATE}.tmp /tmp/${PROJECT}_errors_${DATE}
integer LENGTH
LENGTH=`du -sk /tmp/${PROJECT}_errors_${DATE} | awk '{print $1}'`
if [ "$LENGTH" != 0 ]
then
	echo "\nWARNING: see /tmp/${PROJECT}_errors_${DATE} for errors  !!"
	echo "\nOperation terminated with warnings" 
else
	rm /tmp/${PROJECT}_errors_${DATE}	
	echo "Operation Successful" 

fi

echo ""
exit

