#!/bin/ksh 
############################################################################
#*
#* SCRIPT      : Check_account.ksh
#* ------------
   	VERSION="1.1"
#* ------------
#* AUTHOR      : P.PONCELET
#* ------------
#* DATE        : 28/05/2001
#* ------------
#* DESCRIPTION :   
#* ------------  Parameters:
#*
############################################################################
 
# Nom du script shell
#-----------------------------------------------------------------------------
#
PGM=`basename $0`

# Trace de l'execution
#-----------------------------------------------------------------------------
#
REP_LOG=/tmp
FIC_LOG=`echo $PGM | cut -f 1 -d "."`.log
#exec </dev/null >>$REP_LOG/$FIC_LOG 2>&1

# Langue
# ------
export LANG=C

# Parameters
# ----------
TYPE=$1
ACCOUNT=$2
VERBOSE=$3
SUBDIRECTORY=$4

# Variable
# --------
DATE=`date +%Y%m%d%H%M`
REP_HOME=/usr/var/ase/mnt/ASEuser/export/home
REP_HOMECD=/usr/var/ase/mnt/ASEcd/export/usercd
REP_DATA=/usr/var/ase/mnt/ASEdata/export/data
REP_DATACD=/usr/var/ase/mnt/ASEcd/export/datacd
REP_CAOCD=/usr/var/ase/mnt/ASEcd/export/caocd

# Starting verification
# ---------------------
if [ ! -z "${VERBOSE}" ]
then
	echo "\nProcessing Check_account.ksh file (V${VERSION}) for ${ACCOUNT}"
fi

# Parameters test
# ---------------
if [ ! -z "${VERBOSE}" ]
then
	echo "\nVerifying input parameters: \c"
fi
if [ $# -lt 2 ]
then
	echo "\n\nParameters missing" 
	echo "\nUsage: Check_account.ksh [home|data] account_name  [VERBOSE] [SUBDIRECTORY]"
	echo "Example: Check_account.ksh home c100026"
	exit 1;
else
	case ${TYPE} in
	'home')
		REP=${REP_HOME}
		MASK_REP="754"
		;;
	'homecd')
		REP=${REP_HOMECD}
		MASK_REP="754"
		;;
	'data')
		REP=${REP_DATA}
		MASK_REP="700"
		;;
	'datacd')
		REP=${REP_DATACD}
		MASK_REP="700"
		;;
	'caocd')
		REP=${REP_CAOCD}
		MASK_REP="700"
		;;
	*)
		echo "\n\nParameters invalid (${TYPE})"
		exit 1;
		;; 
	esac

	if [ ! -d ${REP}/${ACCOUNT} ]
	then
		echo "\n\nFAILED : Account or project not found on ${REP} !"
		exit 1;
	fi

	REP_ACCOUNT=${REP}/${ACCOUNT}
	ROOT_ACCOUNT=${REP_ACCOUNT}
	OWNER=`ls -ld ${REP}/${ACCOUNT} | awk '{print $3}'`
	GROUP=`ls -ld ${REP}/${ACCOUNT} | awk '{print $4}'`

	if [ ! -z "${VERBOSE}" ]
	then	
		if [ "${VERBOSE}" != "V" ]
		then
			echo "\n\nParameter must be 'V' or '' !!"
			exit 1;
		fi
	fi
	if [ ! -d "${REP_ACCOUNT}/${SUBDIRECTORY}" ]
	then
		echo "\n\nSub-directory ${SUBDIRECTORY} not found on ${REP_ACCOUNT} !!"
		exit 1;
	else
		REP_ACCOUNT=${REP}/${ACCOUNT}/${SUBDIRECTORY}
	fi
fi
if [ ! -z "${VERBOSE}" ]
then
	echo "Ok"
fi

# Overview (if VERBOSE mode used)
# -------------------------------
if [ ! -z "${VERBOSE}" ]
then
	echo "\nOverview:"
	echo "---------"
	echo "\nProject/account name: 	${ACCOUNT}"
	echo "Owner:			${OWNER}"
	echo "Group:			${GROUP}"
	echo "Root directory: 	${ROOT_ACCOUNT}"
	if [ ! -z "${SUBDIRECTORY}" ]
	then
		echo "Sub-directory: 		${SUBDIRECTORY}"
	fi 
fi

# Listing of directory
# --------------------
if [ ! -z "${VERBOSE}" ] 
then
	echo "Setting protection (${MASK_REP}): \c"
fi
# Setting 777 mask for special logs_execution_softs_dsi directory - A.C. 06/08/2001
# --------------------------------------------------------------------------------
if [ ${ROOT_ACCOUNT} = "/usr/var/ase/mnt/ASEdata/export/data/logs_execution_softs_dsi" ]
then
	MASK_REP="777"
fi 
chmod ${MASK_REP} ${ROOT_ACCOUNT}
STATUS=$?
if [ "${STATUS}" != "0" ]
then
	echo "\n\nError setting protection for ${ROOT_ACCOUNT} (${STATUS}) !!"
	exit 1;
else
	if [ ! -z "${VERBOSE}" ] 
	then
		echo "Ok"
	fi
fi

# Listing of directory
# --------------------
if [ ! -z "${VERBOSE}" ]
then
	echo "Searching invalid owner/group files in ${REP_ACCOUNT}: \c"
fi

if [ "${TYPE}" = "home" ] || [ "${TYPE}" = "homecd" ]
then
	find ${REP_ACCOUNT} ! -user ${OWNER} -ls | awk '{print $5 " " $6 " " $11}' > /tmp/${ACCOUNT}_owner.${DATE}
else
	find ${REP_ACCOUNT} ! -user ${OWNER} -ls | awk '{print $5 " " $6 " " $11}' | grep '^[0-9]' > /tmp/${ACCOUNT}_owner.${DATE}
fi

#find ${REP_ACCOUNT} !  -group ${GROUP} -ls | awk '{print $5 " " $6 " " $11}' >> /tmp/${ACCOUNT}_files.${DATE}
find ${REP_ACCOUNT} !  -group ${GROUP} -ls | awk '{print $5 " " $6 " " $11}' > /tmp/${ACCOUNT}_group.${DATE}
cat /tmp/${ACCOUNT}_owner.${DATE} /tmp/${ACCOUNT}_group.${DATE} | sort -u > /tmp/${ACCOUNT}_files.${DATE}

if [ ! -z "${VERBOSE}" ]
then
	echo "Ok"
fi

# Main loop
# ---------
if [ -s /tmp/${ACCOUNT}_files.${DATE} ]
then
	# Processing files and directories
	# --------------------------------
	if [ ! -z "${VERBOSE}" ]
	then
		echo "Processing invalid files and directories:"
	fi
	
	while read ITEM
	do
		FILE_ITEM=`echo "${ITEM}" | awk '{print $3}'`
		GROUP_ITEM=`echo "${ITEM}" | awk '{print $2}'`
		if [ "${GROUP_ITEM}" = "${GROUP}" ]
		then
			FLAG_GROUP=false
		else	
			FLAG_GROUP=true
		fi

		OWNER_ITEM=`echo "${ITEM}" | awk '{print $1}'`
		if [ "${OWNER_ITEM}" = "${OWNER}" ] 
		then
			FLAG_OWNER=false
		else
			FLAG_OWNER=true
		fi

		if [ "${FLAG_OWNER}" = "true" ] && [ "${FLAG_GROUP}" = "true" ]
		then
			echo "Modifying owner/group ${FILE_ITEM}"
			chown -h ${OWNER}:${GROUP} "${FILE_ITEM}"	
			STATUS=$?
			if [ "${STATUS}" != "0" ]
			then
				echo "\n\nError modifying owner/group of ${FILE_ITEM} (${STATUS})"
				exit 1;
			else
				echo "Old owner: ${OWNER_ITEM}, new owner: ${OWNER}, old group: ${GROUP_ITEM}, new group ${GROUP}, file: ${FILE_ITEM}" >> /tmp/${ACCOUNT}_files_modified.${DATE}
			fi
		else

			if [ "${FLAG_OWNER}" = "true" ]
			then
				echo "Modifying owner ${FILE_ITEM}"
				chown -h ${OWNER} "${FILE_ITEM}"	
				STATUS=$?
				if [ "${STATUS}" != "0" ]
				then
					echo "\n\nError modifying owner of ${FILE_ITEM} (${STATUS})"
					exit 1;
				else
					echo "Old owner: ${OWNER_ITEM}, new owner: ${OWNER}, file: ${FILE_ITEM}" >> /tmp/${ACCOUNT}_files_modified.${DATE}
				fi
			fi

			if [ "${FLAG_GROUP}" = "true" ]
			then
				echo "Modifying group ${FILE_ITEM}"
				chgrp -h ${GROUP} "${FILE_ITEM}"	
				STATUS=$?
				if [ "${STATUS}" != "0" ]
				then
					echo "\n\nError modifying group of ${FILE_ITEM} (${STATUS})"
					exit 1;
				else
					echo "Old group: ${GROUP_ITEM}, new group: ${GROUP}, file: ${FILE_ITEM}" >> /tmp/${ACCOUNT}_files_modified.${DATE}
				fi
			fi
		fi

	done < /tmp/${ACCOUNT}_files.${DATE}
fi

rm /tmp/${ACCOUNT}_owner.${DATE} /tmp/${ACCOUNT}_group.${DATE} /tmp/${ACCOUNT}_files.${DATE}

if [ ! -z "${VERBOSE}" ]
then
	echo "Successfully completed\n"
fi
