#!/usr/bin/ksh
#set -x
#
#=============================================================================
# Nom            :      change_acl.ksh
# Projet         :      
#
# Repertoires    :      script -> /usr/local/bin
#
# Cree le        :      28/03/2002
# Version        :      0.1
# Cree par       :      Jean Philippe BOCQUENET
# Objet          :
# Appel(s)       :      commande en ligne
# Environnements :      HP-UX 9.05 10.20
#                :      Solaris 5.3 5.4 5.5 5.5.1 5.6
#                :      SunOs 4.1.3
#                :      Aix 3.x 4.2 4.3
#                :      Irix 5.3
#                :      Linux 2.2 2.4
# Parametre(s)   :      $1=repertoire a modifier, $2=ACL a ajouter, $3=ACL a enlever
# Codes retour   :      = 0 => OK
#                :      = 1 => KO
#=============================================================================
# Modification(s)
#-----------------------------------------------------------------------------
# JJ/MM/AA - QUI
#       QUOI
# 
#=============================================================================
 
#=============================================================================
# Initialisation des variables utilisees dans le script
# 
# Code retour   : 0 => OK
#               : 1 => KO
#=============================================================================
 
DIRECTORY=$1
CHOIX=$2
TYPEOWNER1=$3
USER_A_REMPLACER=$4
DROITSOWNER1=$5
TYPEOWNER2=$6
USER_DE_REMPLACEMENT=$7
DROITSOWNER2=$8

#=============================================================================
# Nom du script shell
#=============================================================================
PGM=`basename $0`
 
#=============================================================================
# Test la presence des variables a l'entree
#=============================================================================
if [ $# -lt 5 ]
then
     echo "\n\t Utilisation:   ${PGM} repertoire_ou_fichier -r|a|e -u|g ancien_user droits-rwx -u|g nouveau_user droits-rwx"
     echo "\n\t                -r = replace, -a = ajoute, -e = erase\n"
     echo "\n\t Exemple:       ${PGM} /outils/test -r -u c09969 rwx -g mgr_st r-x\n"
     exit 1;
fi
 
# Trace de l'execution
#=============================================================================
REP_LOG=${TEMP}
FIC_LOG=`echo ${PGM} | cut -f 1 -d "."`.log
#exec </dev/null >>${REP_LOG}/${FIC_LOG} 2>&1


#=============================================================================
# Redefinition des variables User et Group
#=============================================================================

case ${TYPEOWNER1} in
-u)	OWNER1="user"
	break
	;;
-g)	OWNER1="group"
        break
        ;;
-o)     OWNER1="other"
        break
        ;;
*)	print "\n\n Valeur incorrecte.\n\n"
	exit 1 
        ;;
esac

if [ ${CHOIX} = "-r" ]
then
	case ${TYPEOWNER2} in
		-u)     OWNER2="user" 
        		break
        		;;
		-g)     OWNER2="group"
        		break
        		;;
		-o)     OWNER2="other"
        		break
        		;;
		*)      print "\n\n Valeur incorrecte.\n\n"
        		exit 1
        		;;
	esac 
fi

if [ ${CHOIX} = "-a" ]
then
	TYPEOWNER2=${TYPEOWNER1}
	OWNER2=${OWNER1}
	USER_DE_REMPLACEMENT=${USER_A_REMPLACER}
	USER_A_REMPLACER="#"
	DROITSOWNER2=${DROITSOWNER1}
fi
 
#=============================================================================
# Remplacement d'un utilisateur par un autre
#=============================================================================

find ${DIRECTORY} -print | while read ITEM
	do

	   if getacl "${ITEM}" | egrep -v "# owner:|# group:|# file:" | grep ${USER_A_REMPLACER} > /dev/null 2>&1
	   then
		if [ -d "${ITEM}" ]
		then
		# Appliquer les ACL sur un directory ---------------

                        # Ajout d'un user
			if [[ "${CHOIX}" = "-a" || "${CHOIX}" = "-r" ]]
			then
				setacl -u ${OWNER2}:${USER_DE_REMPLACEMENT}:${DROITSOWNER2} "${ITEM}"
				setacl -D -u ${OWNER2}:${USER_DE_REMPLACEMENT}:${DROITSOWNER2} "${ITEM}"
				setacl -d -u ${OWNER2}:${USER_DE_REMPLACEMENT}:${DROITSOWNER2} "${ITEM}"
			fi

                        # Destruction d'un user
			if [[ "${CHOIX}" = "-e" || "${CHOIX}" = "-r" ]]
			then
				setacl -x ${OWNER1}:${USER_A_REMPLACER}:${DROITSOWNER1} "${ITEM}"
				setacl -D -x ${OWNER1}:${USER_A_REMPLACER}:${DROITSOWNER1} "${ITEM}"
				setacl -d -x ${OWNER1}:${USER_A_REMPLACER}:${DROITSOWNER1} "${ITEM}"
			fi

		else
		# Applique les ACL sur un fichier ------------------

                        # Ajout d'un user
                        if [[ "${CHOIX}" = "-a" || "${CHOIX}" = "-r" ]]
                        then
				setacl -u ${OWNER2}:${USER_DE_REMPLACEMENT}:${DROITSOWNER2} "${ITEM}"
			fi
 
                        # Destruction d'un user
                        if [[ "${CHOIX}" = "-e" || "${CHOIX}" = "-r" ]]
                        then
                        	setacl -x ${OWNER1}:${USER_A_REMPLACER}:${DROITSOWNER1} "${ITEM}"
			fi
		fi
	   fi
#	printf "%b" "."
	echo "${ITEM}"
	done

#
# eof
#=============================================================================
