#!/usr/bin/ksh 
#
# FILE
# ====
#		
# /sys_admin/script_admin/create_usr.ksh
#		
# PURPOSE
# =======
# This script is designed to create one or more NIS' user account belonging to the same group.
# It replaces the older version written by Ph. MEHL, whose it draws his inspiration from.
#
# NIS group can be created too if no name matches an existing one. Host taking in the user's
# home directory can be either ASEuser (default), cheiron, riviera, opale or ternate.
#
# DATA file structure
# ===================
#	field 1 : user's login name
#	field 2 : user's real name and informations
#	field 3 : user's quota in Mb
#	fields are colon (:) separated
#	Example : c110635:Denis C-Combe (7753) DSI:50
#
# RELEASE/AUTHOR
# ==============
#
# 18/10/2000 : D.C-Combe  - Beta version
# 26/10/2000 : D.C-Combe  - Version 1.0 first set in circulation 
# 27/10/2000 : D.C-Combe  - Version 1.5 some minor issues fixed 
# 31/10/2000 : D.C-Combe  - Version 1.6 Goup detection bug fixed and tag added
# 05/01/2001 : D.C-Combe  - Version 2.0 Prise en compte de la norme SIA
# 27/03/2001 : P.Poncelet - Version 2.1 ajout du tri des MAPS passwd et group
# 20/06/2001 : D.C-Combe  - Version 2.2 Taking into account new primary group policy
# 25/06/2001 : D.C-Combe  - Version 2.3 Trap added
# 27/06/2001 : P.Poncelet - Version 2.4 Script is now localized in /sys_admin/scripts_admin
# 27/08/2001 : D.C-Combe  - Version 2.5 Opale target host added
# 18/10/2001 : D.C-Combe  - Version 2.6 Ident_seq function added
#
################################################

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> FUNCTION'S STATEMENTS AREA - BEGINNING

# ===============
# Check_logname FUNCTION
# This function checks whether or not the logname given in parameter
# is present in the NIS passwd map file
# ===============
Check_logname()
{
        # NIS user logname must be UNknown
        # Option -w categorize a c00000 and a c00000e user
        # ------------------------------------------------
        if [ "$(ypcat passwd | grep -w $1)" ]
        then
                # NIS user's logname exists
                # -------------------------
                return 0
        fi
 
        # NIS user's logname does not exist
        # ---------------------------------
        return 1
}

# ===============
# Set_env FUNCTION
# To set users a generic environment
# ===============
Set_env()
{
	# Generic environment setting
	# ---------------------------
	print "Setting user's generic environment to ${logname}... \c"

	# .profile
	# --------
	if [ -f "${envdir}/${target_host}/.profile" ]
	then
		rcp ${envdir}/${target_host}/.profile ${target_host}:${root_dir}/${logname}
	fi

	# .Xdefaults
	# ----------
	if [ -f "${envdir}/${target_host}/.Xdefaults" ]
	then
		rcp ${envdir}/${target_host}/.Xdefaults ${target_host}:${root_dir}/${logname}
	fi

	# .mwmrc
	# ------
	if [ -f "${envdir}/${target_host}/.mwmrc" ]
	then
		rcp ${envdir}/${target_host}/.mwmrc ${target_host}:${root_dir}/${logname}
	fi

	# .openwin-init
	# -------------
	if [ -f "${envdir}/${target_host}/.openwin-init" ]
	then
		rcp ${envdir}/${target_host}/.openwin-init ${target_host}:${root_dir}/${logname}
	fi

	rsh -n ${target_host} /usr/bin/touch ${root_dir}/${logname}/.rhosts
	rsh -n ${target_host} chown -R ${logname}:${groupname} ${root_dir}/${logname}
	rsh -n ${target_host} chown root ${root_dir}/${logname}/.rhosts

	print "done"
	return
}

# ===============
# Finish_doing FUNCTION
# To end with a flourish
# ===============
Finish_doing()
{
	print "\n########## STOP RUNNING : $(date)" >> ${logdir}/${logfile}
	tail -2000 ${logdir}/${logfile} | sed -e 's/\[.m//g' > ${wkgdir}/${wkgfile}
	mv ${wkgdir}/${wkgfile} ${logdir}/${logfile}

	cp ${logdir}/${logfile} ~c110635
	rm ${wkgdir}/${lockfile}

	return
}

# ===============
# Trap_sub FUNCTION
# <CTRL C> interception routine
# ===============
Trap_sub()
{
	print "\n???<CTRL C> : Procedure cancelled by operator...\n" | ${logcde}
	Finish_doing
	exit
}

# ===============
# Ident_seq FUNCTION
# Who is doing what?
# ===============
Ident_seq()
{
	disp=$(print ${DISPLAY} | awk -F: '{print $1}')
	
	# Is DNS available?
	# -----------------
	if [ -s "/etc/resolv.conf" ]
	then
		sess=$(nslookup ${disp} | awk '{if (NR == 4) print $2}')
		addr=$(nslookup ${disp} | awk '{if (NR == 5) print $2}')
	else
		sess="Not available"
		addr=${disp}
	fi
}

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> FUNCTION'S STATEMENTS AREA - END
#
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> PROGRAM'S STATEMENTS AREA - BEGINNING

# Ensure command return accuracy
# ------------------------------
export LANG=C
VER="2.6"

# Internal useful statements
# --------------------------
modname=$(basename ${0%.*})
wkgfile=${modname}.$$
logfile=${modname}.log
datafile=${modname}.dat
wkghome=${modname}.hom
lockfile=${modname}.lock

# Useful path
# -----------
wkgdir="/tmp"
logdir="/usr/tmp"
srcdir="/sys_admin/scripts_admin"
envdir="${srcdir}/data_util"

homdir="/home"
mildir="/export/home_secure"
altdir="/export/home"
stddir="/usr/var/ase/mnt/ASEuser/export/home"

# NIS' statements 
# ---------------
nissrcdir="/etc/nis"
niscdedir="/var/yp"
nissavdir="/etc/nis/sav"

# Default variable assignation
# ----------------------------
target_host="ASEuser"
shell="/usr/bin/ksh"
password="IqVT6tYPe/z4w"		# $secure$
logcde="tee -a ${logdir}/${logfile}"

# Typeset variables
# -----------------
typeset -l ack

integer uid
integer gid
integer next_gid
integer maxgid=60000
integer old_stepgid=100
integer new_stepgid=1000
integer stepuid=1
integer nb_entries
integer delta
integer low_pjt=3000
integer high_pjt=3999
integer n

# Flag variables
# --------------
newgrpflag=true

# Text effect
# -----------
B="[1m"
S="[4m"
I="[7m"
E="[7m[1m"
N="[0m"

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> PROGRAM'S STATEMENTS AREA - END

clear
banner "${modname}"
banner "ver. ${VER}"

# Who act?
# --------
Ident_seq

# We check the src NIS file 'passwd' & 'group' validity
# -----------------------------------------------------
if [ ! -f "${nissrcdir}/passwd" ]
then
        print "\n???ABORTED : Source NIS' file <${nissrcdir}/passwd> does not exist...\n"
        exit
fi
 
if [ ! -f "${nissrcdir}/group" ]
then
        print "\n???ABORTED : Source NIS' file <${nissrcdir}/group> does not exist...\n"
        exit
fi

# Only one procedure can be running at a given time
# -------------------------------------------------
if [ -f "${wkgdir}/${lockfile}" ]
then
	print "\n???ABORTED : Process locked by another user...\n"
	exit
else
	touch ${wkgdir}/${lockfile}
	touch ${wkgdir}/${wkgfile}
	print "\n########## (${modname} v${VER}) START RUNNING : $(date)" >> ${logdir}/${logfile}
	print "           Current terminal : ${sess} - address : ${addr}" >> ${logdir}/${logfile}
fi

# trap ^C
# -------
trap Trap_sub INT QUIT TERM TSTP

# Available TARGET HOST
# ---------------------
print "\n${E}Choose from these following servers the target host to take in the user's HOME DIRECTORY${N}\n"
print "\t+ ASEuser (default)"
print "\t+ cheiron"
print "\t+ riviera"
print "\t+ opale"
print "\t+ ternate\n"

# This loop allows "target host" choice
# -------------------------------------
while read rep?"Enter TARGET HOST to take in the user's HOME DIRECTORY [${B}${target_host}${N}]? : "
do
	case ${rep:=${target_host}} in

	"ASEuser")
		root_dir=${stddir}					# /usr/var/ase/mnt/ASEuser/export/home
		break
		;;

	"cheiron" | "riviera"  | "ternate" | "opale")
		target_host=${rep}
		root_dir=${altdir}					# /export/home
		break
		;;
	*)
		print "\n\t!!!WARNING : Entry <${B}${rep}${N}> is not an available choice...\n"
		continue
		;;
	esac
done

# First, user must belongs to a group
# -----------------------------------
while read groupname?"Choose a membership's groupe name? (? to display the existing name list, leave blank to quit) : "
do
	if [ "${groupname}" = '?' -o "${groupname}" = '*' ]
	then
		ypcat group | awk -F: '{if ($3 < '${low_pjt}' || $3 > '${high_pjt}') print $1"     	"$3}' | sort -n -k 2 | more
		continue
	fi

	# Is it a valid entry?
	# --------------------
	if [ ! "${groupname}" ]
	then
		# NO : Blank field 
		# ----------------
		print "\n!!!STOP : Action canceled by operator...\n" | ${logcde}

		Finish_doing
		exit
	else
		# gid lower significant value
		# ---------------------------
		let low_bracket=100

		# Higher value
		# ------------
		for val in $(ypcat group | awk -F: '{print $3}' | sort -n)
		do
			high_bracket=${val}
		done

		# Groupname cannot be a name used as template for quota allocation
		# ----------------------------------------------------------------
		case ${groupname} in

		"etg_"*)
			print "\n\t!!!UNAVAILABLE : This (part of) name is reserved for group quota allocation templates...\n"
			continue
			;;
		esac

		# YES : Checking the groupname's length. This one must be of 8 characters max
		# due to the ACLs applying rules
		# ------------------------------
		if [ ${#groupname} -gt 8 ]
		then
			print "\n\t???FAILED : NIS' groupname length too long, ${S}8${N} char max expected...\n"
			continue
		fi

		# Does the groupname exist?
		# -------------------------
		if [ "$(ypcat group | grep -w ${groupname})" ]
		then
			# YES : What Number of occurrences is found, the only value
			# accepted is 1 otherwise there is a problem
			# ------------------------------------------
			set $(ypcat group | grep -w ${groupname} | awk '{print NR}')

			# Testing the number of returned occurences
			# -----------------------------------------
			if [ "$#" -gt 1 ]
			then
				print "\n???ABORTED : ${B}$#${N} instances of ${groupname} found in NIS group map..." | ${logcde}
				print "             Map's issue must be fixed first...\n"

				Finish_doing
				exit
			fi

			# Found name is okay
			# ------------------
			grpentry=$(ypcat group | grep -w ${groupname})
			gid=$(print ${grpentry} | awk -F: '{print $3}')

			# What is the next valid gid?
			# ---------------------------
			for val in $(ypcat group | awk -F: '{if ($3 > '${gid}') print $3}' | sort -n)
			do
				next_gid=${val}
				break
			done

			# Checking the possibility of being a secondary group
			# ---------------------------------------------------
			cat ${nissrcdir}/passwd | awk -F: '{print $4}' > ${wkgdir}/${wkgfile}

			# Is gid classified in NIS source passwd file?
			# --------------------------------------------
			if [ ! "$(grep -w ${gid} ${wkgdir}/${wkgfile})" ]
			then
				# No membership found for this group, this is a secondary group
				# -------------------------------------------------------------
				print "\n\t!!!WARNING : <${B}${groupname}${N}> ID ${gid} seems to be a PROJECT group..."

				# Are there a lot of members belonging to this group
				# --------------------------------------------------
				if [ "$(grep -w ${gid} ${nissrcdir}/group | awk -F: '{print $4}')" = "" ]
				then
					# Not at all
					# ----------
					print "\t             No member has been found for this group"
				else
					# Okay
					# ----
					print "\t             At least one member belongs to this group"
				fi

				print "\t             IT IS NOT RECOMMENDED TO MAKE THIS GROUP A PRIMARY GROUP...\n"
			fi

			newgrpflag=false
		else
			# NO : We are about to allocate a new group ID
			# From 100 to 10000 with a step of 100
			# From 10000 to 60000 with a step of 1000
			# ---------------------------------------
			cat ${nissrcdir}/group | awk -F: '{if ($3 < '${low_pjt}' || $3 > '${high_pjt}') print $3}' | sort -n > ${wkgdir}/${wkgfile}

			# Searching a free GID value
			# --------------------------
			while [[ "${low_bracket}" -le "${high_bracket}" ]]
			do
				if [ ! "$(grep -w ${low_bracket} ${wkgdir}/${wkgfile})" ]
				then
					# Possible free GID found
					# -----------------------
					let gid=${low_bracket}
					grpentry="${groupname}::${gid}:"
					break
				fi

				# Work out the search increment for GID
				# -------------------------------------
				if [ "${low_bracket}" -lt 10000 ]
				then
					# from 100 to 100
					# ---------------
					let low_bracket=${low_bracket}+${old_stepgid}
				else
					# from 1000 to 1000
					# -----------------
					let low_bracket=${low_bracket}+${new_stepgid}
				fi
			done

			# Upper limit has been reached without possible free GID
			# ------------------------------------------------------
			if [ "${gid}" -eq 0 -a "${high_bracket}" -lt 10000 ]
			then
				let gid=${high_bracket}+${old_stepgid}
			elif [ "${gid}" -eq 0 -a "${high_bracket}" -ge 10000 ]
			then
				let gid=${high_bracket}+${new_stepgid}
			fi

			# New GID loop agreement
			# ---------------------- 
			while read rep?"Possible new GID = ${B}${gid}${N} type RETURN to accept, enter a value or '?' to display already used GID : "
			do
				# Display existing GID
				# --------------------
				if [ "${rep}" = '?' ]
				then
					cat ${wkgdir}/${wkgfile} | more
					continue
				fi

				# Out of limit
				# ------------
				if [ "${rep:=${gid}}" -gt 60000 ]
				then
					print "\n\t!!!WARNING : Value for GID cannot be greater than ${maxgid}...\n"
					continue
				else
					# GID already allocated
					# ---------------------
					if [ "$(grep -w ${rep} ${wkgdir}/${wkgfile})" ]
					then
						print "\n\t!!!WARNING : GID value ${B}${rep}${N} already used...\n"
						continue
					fi
				fi

				break
			done

			let gid=${rep}
			grpentry="${groupname}::${gid}:"

			# What is the next valid gid?
			# ---------------------------
			for val in $(ypcat group | awk -F: '{if ($3 > '${gid}') print $3}' | sort -n)
			do
				next_gid=${val}
				break
			done
		fi

		# Confirm membership groupname  y/n
		# ---------------------------------
		if [ "${newgrpflag}" = "true" ]
		then
			read ack?"User will be a member of the NEW GROUP <${B}${groupname}${N}> GID <${B}${gid}${N}>? [y]/n : "
		else
			read ack?"User will be a member of the EXISTING GROUP <${B}${groupname}${N}>? [y]/n : "
		fi

		# Checking the answer
		# -------------------
		if  [ ${ack:=y} = 'n' ]
		then
			print "\n\t!!!BREAK : To err is human...\n"
			newgrpflag=true
			continue
		fi 
	fi

	break
done

# This is not a new group
# -----------------------
if [ "${newgrpflag}" = "false" ]
then
	# YES It's not :  We treat exception rules
	# If groupname is strucm or struce, TARGET HOST is switched to "ternate" 
	# ----------------------------------------------------------------------
	case ${groupname} in

	"strucm")
		if [ "${target_host}" != "ternate" ]
		then
			print "\n!!!WARNING : TARGET HOST <${B}${target_host}${N}> and GROUPNAME <${B}${groupname}${N}> mismatch..."
			print "             TARGET HOST has been switched to <${B}ternate${N}>\n"
		fi

		target_host="ternate"
		root_dir=${mildir}						# /export/home_secure
		suffix='m'
		;;

	"struce")
		if [ "${target_host}" != "ternate" ]
		then
			print "\n!!!WARNING : TARGET HOST <${B}${target_host}${N}> and GROUPNAME <${B}${groupname}${N}> mismatch..."
			print "             TARGET HOST has been switched to <${B}ternate${N}>\n"
		fi

		target_host="ternate"
		suffix='e'
		;;

	*)
		suffix=''
		;;
	esac
fi

# Time is up to get new user's specifications
# Users can be entered from a pre-defined file list or by an interactive method
# -----------------------------------------------------------------------------
while read ack?"Do you want to proceed from data file <${B}${srcdir}/${datafile}${N}> to enter user's specifications? y/[n]/? : "
do
	if  [ ${ack:=n} = '?' ]
	then
		print "\n${E}DATA file structure${N}"
		print "\t${B}field 1${N} : user's login name"
		print "\t${B}field 2${N} : user's real name and informations"
		print "\t${B}field 3${N} : user's quota allocation in Mb (default is 50Mb) for ASEUSER and cheiron"
		print "\tfields are colon (:) separated\n"
		print "${B}Example${N} : c110635:Denis C-Combe (7753) DSI:200\n"
		continue
	fi
	
	break
done

print "\n"

# Testing the given answer
# ------------------------
if  [ ! ${ack:=n} = 'y' ]
then
	# NO : INTERACTIVE METHOD, data file is cleaned up
	# ------------------------------------------------
	cp /dev/null ${srcdir}/${datafile}

	# Entry loop for users being added
	# --------------------------------
	while read field1?"Enter user's logname? (Leave empty when done) : "
	do
		# Is userid null?
		# ---------------
		if [ ! "${field1}" ]
		then
			# Is the size of the data file <> 0
			# --------------------------------- 
			if [ ! -s "${srcdir}/${datafile}" ]
			then
				print "\n???ABORTED : At least one ${B}${groupname}${N}'s membership is needed...\n" | ${logcde}

				Finish_doing
				exit
			fi

			break
		fi

		# User logname must be UNknown
		# ----------------------------
		if Check_logname ${field1}
		then
			# Known logname encountered
			# -------------------------
			print "!!!\n\tWARNING : The user's logname <${B}${field1}${N}> already exists...\n"
			continue
		else
			# Specific rule handling
			# ----------------------
			last_char=$(print ${field1} | awk '{print substr($1,length($1),1)}')

			case ${last_char} in

			[0-9])
				field1=${field1}"${suffix}" > /dev/null 2>&1
				
				# Logname has been modified, a new control is necessary
				# -----------------------------------------------------
				if Check_logname ${field1}
				then
					# Known logname encountered
					# -------------------------
					print "!!!\n\tWARNING : The user's logname <${B}${field1}${N}> already exists...\n"
					continue
				fi
				;;

			"m" | "e")
				if [ "${suffix}" = "" ]
				then
					print "\n\t!!!WARNING : Ending character <${B}${last_char}${N}> may be unrelevant in that case...\n"
				fi

				if [ "${groupname}" = "strucm" -a ${last_char} = 'e' ]
				then
					print "\n\t!!!FAILED : Unrelevant ending character <${B}${last_char}${N}>, <${B}${suffix}${N}> is expected...\n"
					continue
				fi

				if [ "${groupname}" = "struce" -a ${last_char} = 'm' ]
				then
					print "\n\t!!!FAILED : Unrelevant ending character <${B}${last_char}${N}>, <${B}${suffix}${N}> is expected...\n"
					continue
				fi
				;;
			esac
					
			# Entry loop for user's real name
			# -------------------------------
			while read field21?"Enter user's real name? : "
			do
				if [ ! "${field21}" ]
				then
					print "\n\t!!!WARNING : User's real name needful...\n"
					continue
				else
					break
				fi
			done

			# Miscellanous nfo
			# ----------------
			read field22?"Phone number, department...? : "

			# Entry for user's quota
			# ----------------------
			if [ "${target_host}" = "ASEuser" -o "${target_host}" = "cheiron" ]
			then
				read field3?"Enter user's quota expected? 10/[50]/200 : "

				case ${field3:=50} in
	
				10 | 50 | 200)
					;;
				*)
					let field3=50
					;;
				esac
			else
				let field3=0
			fi
					
			# New user entry setting
			# ----------------------
			print "\n"
			read ack?"${field1}:${field21} ${field22}:${field3}Mbquotas correct? [y]/n : "
			
			if [ ${ack:=y} = 'y' ]
			then
				print "${field1}:${field21} ${field22}:${field3}" >> ${srcdir}/${datafile}
			fi

			print "\n"
			continue
		fi
	done
else
	# YES : NON INTERACTIVE method, is the size of the data file <> 0
	# --------------------------------------------------------------- 
	if [ ! -s "${srcdir}/${datafile}" ]
	then
		print "\n???ABORTED : File <${B}${srcdir}/${datafile}${N}> is empty...\n" | ${logcde}

		Finish_doing
		exit
	fi

	# Controlling the data file structure and reliability
	# ---------------------------------------------------
	print "\n${E}Data file's structure and reliability control sequence${N}\n"

	while read line
	do
		field1=$(print ${line} | awk -F: '{print $1}')
		field2=$(print ${line} | awk -F: '{print $2}')
		field3=$(print ${line} | awk -F: '{print $3}')
	
		# Checking for empty fields
		# -------------------------
		if [ "${field1}" = "" -o "${field2}" = "" -o "${field3}" = "" ]
		then
			print "Checking entry > ${line}"
			print "\n\t???ABORTED : Empty field(s) encountered in the data file...\n" | ${logcde}

			Finish_doing
			exit
		fi

		# User's quota must be 10, 50 or 200 Mo
		# -------------------------------------
		if [ "${target_host}" = "ASEuser" -o "${target_host}" = "cheiron" ]
		then
			case ${field3} in

			10 | 50 | 200)
				;;
			*)
				print "Checking entry > ${line}"
				print "\n\t???ABORTED : User's quota must be 10, 50 or 200Mb...\n" | ${logcde}

				Finish_doing
				exit
				;;
			esac
		fi
		
		# User logname must be UNknown
		# ----------------------------
		if Check_logname ${field1}
		then
			# Known logname encountered
			# -------------------------
			print "Checking entry > ${line}"
			print "\n\t???ABORTED : The user's logname <${B}${field1}${N}> already exists in NIS' map...\n" | ${logcde}

			Finish_doing
			exit
		fi

		# Specific rule handling
		# ----------------------
		last_char=$(print ${field1} | awk '{print substr($1,length($1),1)}')

		case ${last_char} in

		[0-9])
			if [ "${suffix}" != "" ]
			then
				print "Checking entry > ${line}"
				print "\n\t???ABORTED : Unrelevant ending character <${B}${last_char}${N}>, <${B}${suffix}${N}> expected...\n" | ${logcde}

				Finish_doing
				exit
			fi
			;;

		"m" | "e")
			if [ "${suffix}" = "" ]
			then
				print "Checking entry > ${line}"
				print "\n\t!!!WARNING : Ending character <${B}${last_char}${N}> may be unrelevant in this case...\n" | ${logcde}
			fi

			if [ "${groupname}" = "strucm" -a ${last_char} = 'e' ]
			then
				print "Checking entry > ${line}"
				print "\n\t???ABORTED : Unrelevant ending character <${B}${last_char}${N}>, <${B}${suffix}${N}> expected...\n" | ${logcde}

				Finish_doing
				exit
			fi

			if [ "${groupname}" = "struce" -a ${last_char} = 'm' ]
			then
				print "Checking entry > ${line}"
				print "\n\t???ABORTED : Unrelevant ending character <${B}${last_char}${N}>, <${B}${suffix}${N}> expected...\n" | ${logcde}

				Finish_doing
				exit
			fi
			;;
		esac

	done < ${srcdir}/${datafile}
fi

# Common data file checking
# -------------------------
print "\n${E}Data redundancy check${N}"

while read line
do
	field1=$(print ${line} | awk -F: '{print $1}')

	# Checking for possible duplicated entries in the data file
	# ---------------------------------------------------------
	set $(grep -w ${field1} ${srcdir}/${datafile} | awk '{print NR}')

	# Testing the number of parameters set
	# ------------------------------------
	if [ "$#" -gt 1 ]
	then
		print "\n???ABORTED : ${B}$#${N} occurrences of ${B}${field1}${N} found in <${B}${srcdir}/${datafile}${N}>...\n" | ${logcde}
		print "             File's issue must be fixed first...\n"

		Finish_doing
		exit
	fi

done < ${srcdir}/${datafile}

nb_entries=$(cat ${srcdir}/${datafile} | wc -l)

# Finding an appropriate UID for each user
# ----------------------------------------
lower_bound=${gid}

# Determining the upper value
# ---------------------------
if [ "${next_gid}" -eq 0 ]
then
	let upper_bound=${gid}+${new_stepgid}
else
	let delta=${next_gid}-${lower_bound}

	if [ "${delta}" -lt 2 ]
	then
		print "\n???ABORTED : Not enough free UIDs are available...\n" | ${logcde}

		Finish_doing
		exit
		
	# Between 2 and 100 possible UIDs, we set 'upper_bound' to the next_gid found
	# ---------------------------------------------------------------------------
	elif [ "${delta}" -le 100 ]
	then
		let upper_bound=${next_gid}

	# Between 101 and 999 possible UIDs we set 'upper_bound' to 'lower_bound'+100
	# ---------------------------------------------------------------------------
	elif [ "${delta}" -gt 100 -a "${delta}" -lt 1000 ]
	then
		let upper_bound=${gid}+${old_stepgid}
	else
		# Upper than 999, we set 'upper_bound' to 'lower_bound'+1000
		# ----------------------------------------------------------
		let upper_bound=${gid}+${new_stepgid}
	fi
fi

# Log file Header recording
# -------------------------
print "\nNIS' group name ...........: ${groupname}"  | ${logcde}
print "NIS' group ID .............: ${gid}"          | ${logcde}
print "Lower range for valid UID .: ${lower_bound}"  | ${logcde}
print "Higher range for valid UID : ${upper_bound}"  | ${logcde}
print "New group .................: ${newgrpflag}"   | ${logcde}
print "Home directory location ...: ${target_host}"  | ${logcde}
print "Root directory ............: ${root_dir}"     | ${logcde}
print "Home directory ............: ${homdir}"       | ${logcde}
print "Nb of entries .............: ${nb_entries}\n" | ${logcde}

print "\n${E}Free UIDs search sequence${N} \c"

# Getting bound of possibilities
# ------------------------------ 
cat ${nissrcdir}/passwd | awk -F: '{if ($3 >= '${lower_bound}' && $3 < '${upper_bound}') print $3}' > ${wkgdir}/${wkgfile}

let index=0

# print ${lower_bound}
# print ${upper_bound}
# print ${index}
# print ${nb_entries}


# Recovery of free uids among possibilities
# -----------------------------------------
while [[ "${lower_bound}" -lt "${upper_bound}" && "${index}" -lt "${nb_entries}" ]]
do
	if [ ! "$(grep -w ${lower_bound} ${wkgdir}/${wkgfile})" ]
	then
		# Free
		# ----
		print "<${lower_bound}>\c"
		let index=${index}+1
		freeuid[${index}]="${lower_bound}"
	else
		print ".\c"
	fi

	let lower_bound=${lower_bound}+${stepuid}
done

# Checking wether or not all new users have an allocated UID
# ----------------------------------------------------------
if [ "${index}" -lt "${nb_entries}" ]
then
	print "\n???ABORTED : ${B}${index}${N} UID(s) available for ${B}${nb_entries}${N} new entrie(s)...\n" | ${logcde}

	Finish_doing
	exit
fi

print "\n\n******************** ${srcdir}/${datafile}'s contents start\n" | ${logcde}
cat ${srcdir}/${datafile} | ${logcde}
print "\n******************** ${srcdir}/${datafile}'s contents stop\n" | ${logcde}

# A chance to stop before modifications take place
# ------------------------------------------------
read ack?"Ready to proceed? [y]/n : "

if [ ! ${ack:=y} = 'y' ]
then
	print "\n???ABORTED : Procedure cancelled by operator...\n" | ${logcde}

	Finish_doing
	exit
fi

print "\n${E}System files pre-processing${N}\n"

# Making a safety copy of files about to be modified
# --------------------------------------------------
cp ${nissrcdir}/group  ${nissavdir}/group.$(date +%Y%m%d)
cp ${nissrcdir}/shadow ${nissavdir}/shadow.$(date +%Y%m%d)
cp ${nissrcdir}/passwd ${nissavdir}/passwd.$(date +%Y%m%d)

# Index and working file initialisation
# -------------------------------------
let index=1

# RAZ of the working file
# -----------------------
cp /dev/null ${wkgdir}/${wkgfile}
print "\n" >> ${logdir}/${logfile}

# Auto_home, group, passwd, shadow and home directories entries' construction
# ---------------------------------------------------------------------------
while read line
do
	logname=$(print ${line} | awk -F: '{print $1}')
	comment=$(print ${line} | awk -F: '{print $2}')
	uid=${freeuid[${index}]}

	print "Updating system's files with ${logname}'s data... \c"

	# passwd string building
	# ----------------------
	passwd_str="${logname}:x:${uid}:${gid}:${comment}:${homdir}/${logname}:${shell}"
	print ${passwd_str} >> ${nissrcdir}/passwd
	cp ${nissrcdir}/passwd ${wkgdir}/${wkgfile}
	sort -n -t : -k 3 ${wkgdir}/${wkgfile} > ${nissrcdir}/passwd

	# shadow string building
	# ----------------------
	shadow_str="${logname}:${password}:10773::::::"
	print "${shadow_str}" >> ${nissrcdir}/shadow

	# auto_home string building
	# -------------------------
	auto_string="${logname}\t\t${target_host}:${root_dir}/${logname}"
	print ${auto_string} > ${wkgdir}/${wkgfile} 

	# Users' home directories creation in 'root_dir'
	# ----------------------------------------------
	rsh -n ${target_host} mkdir ${root_dir}/${logname}     > /dev/null 2>&1
	rsh -n ${target_host} chmod 754 ${root_dir}/${logname} > /dev/null 2>&1

	print "${logname}, UID ${uid} elected" >>  ${logdir}/${logfile}

	let index=${index}+1
	print "done"

done < ${srcdir}/${datafile}

print "\n"

# Up to date group entry is saved at his turn
# -------------------------------------------
if [ "${newgrpflag}" = "true" ]
then
	# YES : We add an entry
	# ---------------------
	print ${grpentry} >> ${nissrcdir}/group
fi

# Sorting NIS MAP group
# ---------------------
sort -n -t : -k 3,3 ${nissrcdir}/group > ${wkgdir}/${wkghome}
mv ${wkgdir}/${wkghome} ${nissrcdir}/group

# If the target host is not ASEuser, the auto_home file must be updated too
# -------------------------------------------------------------------------
if [ "${target_host}" != "ASEuser" ]
then
	# Making a copy of the file about to be modified
	# ----------------------------------------------
	cp ${nissrcdir}/auto_home ${nissavdir}/auto_home.$(date +%Y%m%d)

	# Reading the working file previously constructed
	# -----------------------------------------------
	while read line
	do
		logname=$(print ${line} | awk '{print $1}')
		mopoint=$(print ${line} | awk '{print $2}')

		# Insert a new data line before "*		ASEuser:/export/home/&" pattern
		# -----------------------------------------------------------------------------
		sed '/\*/i\
'${logname}'		'${mopoint}'' ${nissrcdir}/auto_home > ${wkgdir}/${wkghome}

		mv ${wkgdir}/${wkghome} ${nissrcdir}/auto_home

	done < ${wkgdir}/${wkgfile}
fi

# Spreading the NIS' map if I will
# --------------------------------
while read ack?"Are you ready to apply changes? (${B}no${N} resets modified system files to their previous values) [y]/n : "
do
	if [ "${ack:=y}" = '?' ]
	then
		print "\n********* GROUP diff...\n"
		diff ${nissavdir}/group.$(date +%Y%m%d) ${nissrcdir}/group
		print "\n********* PASSWD diff...\n"
		diff ${nissavdir}/passwd.$(date +%Y%m%d) ${nissrcdir}/passwd
		print "\n********* SHADOW diff...\n"
		diff ${nissavdir}/shadow.$(date +%Y%m%d) ${nissrcdir}/shadow

		if [ "${target_host}" != "ASEuser" ]
		then
			print "\n********* AUTO_HOME diff...\n"
			diff ${nissavdir}/auto_home.$(date +%Y%m%d) ${nissrcdir}/auto_home
		fi

		print "\n"

		continue
	fi

	break
done

# Confirm?
# --------
if [ ! "${ack}" = 'y' ]
then
	# I don't
	# -------
        print "\n???WARNING : Resetting modified system's files to their previous value...\c" | ${logcde}

	cp ${nissavdir}/group.$(date +%Y%m%d)  ${nissrcdir}/group
	cp ${nissavdir}/shadow.$(date +%Y%m%d) ${nissrcdir}/shadow
	cp ${nissavdir}/passwd.$(date +%Y%m%d) ${nissrcdir}/passwd

	if [ "${target_host}" != "ASEuser" ]
	then
		cp ${nissavdir}/auto_home.$(date +%Y%m%d) ${nissrcdir}/auto_home
	fi

	# Home directories deletion
	# -------------------------
	while read line
	do
		logname=$(print ${line} | awk -F: '{print $1}')
		rsh -n ${target_host} rmdir ${root_dir}/${logname}

	done < ${srcdir}/${datafile}

	print "done\n" | ${logcde}

	Finish_doing
	exit
fi

# CTRL C and so on forbidden from this point
# ------------------------------------------
trap '' INT QUIT TERM TSTP

# I do confirm
# ------------
print "\n${E}Spreading changes through the NIS' map${N}\n"

# File's tag
# ----------
tag=$(date +%H%M)
mv ${nissavdir}/group.$(date +%Y%m%d)  ${nissavdir}/group.$(date +%Y%m%d)_${tag}
mv ${nissavdir}/shadow.$(date +%Y%m%d) ${nissavdir}/shadow.$(date +%Y%m%d)_${tag}
mv ${nissavdir}/passwd.$(date +%Y%m%d) ${nissavdir}/passwd.$(date +%Y%m%d)_${tag}

# Particular case
# ---------------
if [ "${target_host}" != "ASEuser" ]
then
	mv ${nissavdir}/auto_home.$(date +%Y%m%d) ${nissavdir}/auto_home.$(date +%Y%m%d)_${tag} 
fi

# About to run make command
# -------------------------
sav_path=$(pwd)
cd ${niscdedir}

make

cd ${sav_path}

# Allocating generic environment and quota if needed
# ---------------------------------------------------
print "\n${E}Allocation of user's environment and user's quota${N}\n"

# For all new users
# -----------------
while read line
do
	logname=$(print ${line} | awk -F: '{print $1}')
	quota=$(print ${line} | awk -F: '{print $3}')

	# New user's quota allocation, only on ASEuser and cheiron
	# --------------------------------------------------------
	if [ "${target_host}" = "ASEuser" -o "${target_host}" = "cheiron" ]
	then
		# YES : it's ASEuser or cheiron
		# ----------------------------- 
		print "Allocating ${quota}Mb quota to ${logname}... \c"

		rsh -n ${target_host} /sys_admin/scripts_admin/7upquotaU.ksh ${logname} ${quota}
		print "done"
	fi

	# Setting generic environment except for "opale" which comes from another world
	# -----------------------------------------------------------------------------
	if [ "${target_host}" != "opale" ]
	then
		Set_env
	fi

done < ${srcdir}/${datafile}

print "\nUSER(S) SUCCESSFULLY ADDED" | ${logcde}

# The affair is in the donkey's asshole
# -------------------------------------
Finish_doing

print "Log file ${B}${logfile}${N} available in ${B}${logdir}${N}\n"
print "DON'T FORGET TO LAUNCH \"lsadmin reconfig\" ON MASTER LSF SERVER \"terror\" IF NECESSARY."
