#!/bin/ksh
#********************************************************************
#
# APPLICATION	: Administration generale des serveurs
#
#-----------------------------------------------------------------------------
#
# Source        : purge.scr
#
# Repertoire	: /sys_admin/scripts_admin
#
# Parametres	: Fichier de parametrage : purge.prm FIC_PRM
#
# Description	: Script permettant de supprimer des fichiers de facon parametrable
#		: dir_fic  = repertoire
#		: type_fic = type de fichier
#		: name_fic = noms des fichiers
#		: ret_fic  = duree de retention des fichiers
#
# Script(s) appelant(s) : Lance par le cron tous les soirs.
#
# Script(s) appele(s) :
#
#
#-----------------------------------------------------------------------------
#
# O.S           : HP-UX 9.0 , AIX 3.2, Solaris
#
# Version 	: @(#) Script.ref V:0.1
#
# Historique	:
#
# H. N'kaoua		30/08/96	creation
# H. N'kaoua		13/09/96	modification
#			Remplacement -print par -ls sur les find
# H. N'kaoua            20/12/97        modification
#                       Remplacement -print par -ls sur le find pour les repertoires
# P.Poncelet            01/12/2000      modification
#                       Debug et ajout de controles
#
# S.San Jose            04/03/2002      modification
#                       Ajout prise en cpt des fichiers DBALL et MASTER
#
#-----------------------------------------------------------------------------
#
#
# Nom du script shell
#-----------------------------------------------------------------------------
#
PGM=`basename $0`

#
# Mode debug
#-----------------------------------------------------------------------------
#
#SHELL_DEBUG=1
if [ $SHELL_DEBUG ]
 then
	{ set -v ; set -x; }
fi

# Scrolling the file /tmp/purge.log with the 100 last lines
if [ -f /tmp/purge.log ]
then
        tail -100 /tmp/purge.log > /tmp.purge.log.tail
        mv /tmp.purge.log.tail /tmp/purge.log
fi

#
# Trace de l'execution
#-----------------------------------------------------------------------------
#
REP_LOG=/tmp
FIC_LOG=`echo $PGM | cut -f 1 -d "."`.log
exec </dev/null >>$REP_LOG/$FIC_LOG 2>&1

#
# Les variable globales
#-----------------------------------------------------------------------------
#
IFS=:
FIC_PRM=/sys_admin/scripts_admin/purge.prm

echo "\n********************************************************************************"
echo "`date | cut -d \" \" -f 1-4` -I- Debut de traitement"

#-----------------------------------------------------------------------------
#
# Test existence du fichier de parametrage
#
if [ ! -f "${FIC_PRM}" ]
then	
	echo "`date | cut -d \" \" -f 1-4` -F- Pas de fichier de parametrage, fin de traitement"
	exit 1;
fi

while read dir_fic type_fic name_fic ret_fic
do
	if [ "${type_fic}" = "f" ]
	then
		#
		# Liste et suppression des fichiers
		#
		touch ${dir_fic}/quota* > /dev/null 2>&1 
		echo "`date | cut -d \" \" -f 1-4` -I- Purge fichiers ${name_fic} sous ${dir_fic} (${ret_fic} jrs)"
                if [ "${name_fic}" != "*"  ]
                then
                  find  ${dir_fic} -type ${type_fic} -name "${name_fic}" -atime +${ret_fic} -print -exec rm -rf {} \; 2> /dev/null 
                else
                  # exclusion des .DBALL et .MASTER
		  find  ${dir_fic} \( ! -name "*.DBALL" \) -a \( ! -name "*.MASTER" \) -type ${type_fic} -name "${name_fic}" -atime +${ret_fic} -print -exec rm -rf {} \; 2> /dev/null
                fi
	else 
		if [ "${type_fic}" = "d" ]
		then
			#
			# Liste et suppression des repertoires
			#
			touch ${dir_fic}/quota* > /dev/null 2>&1 
			echo "`date | cut -d \" \" -f 1-4` -I- Purge repertoires ${name_fic} sous ${dir_fic} (${ret_fic} jrs)"
			find  ${dir_fic} -type ${type_fic} -name "${name_fic}" -mtime +${ret_fic} -print -exec rm -rf {} \; 2> /dev/null
		fi
	fi

done < ${FIC_PRM}

echo "`date | cut -d \" \" -f 1-4` -I- Fin de traitement"

