#!/bin/ksh 
############################################################################
#*
#* SCRIPT      : save_quotas.ksh
#* ------------
#* VERSION     : 1.0
#* ------------
#* DESCRIPTION : Sauvegardes des quotas de ASEuser, ASEdata, ASEcd
#* ------------  
#*
############################################################################


# Nom du script shell
#-----------------------------------------------------------------------------
#
PGM=`basename $0`

#
# Mode debug
#-----------------------------------------------------------------------------
#
#SHELL_DEBUG=1
if [ $SHELL_DEBUG ]
 then
        { set -v ; set -x; }
fi

#
# Trace de l'execution
#---------------------
#
REP_LOG=/tmp
FIC_LOG=`echo $PGM | cut -f 1 -d "."`.log
exec </dev/null >>$REP_LOG/$FIC_LOG 2>&1

#
# Variables
# ---------
REP_QUOTAS=/sys_admin/configuration/quotas_ase
REP=$1
DATE=`date +%Y%m%d`

echo "\n********************************************************************************"
echo " `date | cut -d \" \" -f 1-5` -I- Debut de traitement"
if [ $# -lt 1 ]
then
     echo " `date | cut -d \" \" -f 1-5` -F- Utilisation: save_quotas.ksh ASEuser|ASEdata|ASEcd"
     echo " `date | cut -d \" \" -f 1-5` -F- Exemple: save_quotas.ksh ASEuser"
     echo " `date | cut -d \" \" -f 1-5` -I- Fin de traitement"
     exit 1;
fi

# Programme principal
# -------------------

mv ${REP_QUOTAS}/${REP} ${REP_QUOTAS}/${REP}.${DATE} > /dev/null 2>&1

case ${REP} in
'ASEuser')
	echo " `date | cut -d \" \" -f 1-5` -I- Sauvegarde des quotas ${REP}"
	/usr/sbin/vrepquota -a -u > ${REP_QUOTAS}/${REP}
	;;
'ASEdata')
	echo " `date | cut -d \" \" -f 1-5` -I- Sauvegarde des quotas ${REP}"
	/usr/sbin/vrepquota -a -g  > ${REP_QUOTAS}/${REP}
	;;
'ASEcd')
	echo " `date | cut -d \" \" -f 1-5` -I- Sauvegarde des quotas ${REP}"
	/usr/sbin/vrepquota -a -g  > ${REP_QUOTAS}/${REP}
	/usr/sbin/vrepquota -a -u  >> ${REP_QUOTAS}/${REP}
	;;
*)
	echo " `date | cut -d \" \" -f 1-5` -F- Erreur de parametre $1 (ASEuser|ASEdata|ASEcd)"
	exit;
esac

echo " `date | cut -d \" \" -f 1-5` -I- Fin de traitement"
exit 0
